==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/include/gm_page_hash.h <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation			 *
 * Copyright (c) 1999 by Myricom, Inc.					 *
 * All rights reserved.	 See the file `COPYING' for copyright notice.	 *
 *************************************************************************/

#ifndef _GM_page_table_h_
#define _GM_page_table_h_

/****************************************************************
 * Switches
 ****************************************************************/

#define GM_DEBUG_PAGE_HASH 0
#define GM_DEBUG_HASH 0

/****************************************************************
 * Includes
 ****************************************************************/

#include "gm.h"
#include "gm_debug.h"
#include "gm_enable_galvantech_workaround.h"

/****************************************************************
 * Globals
 ****************************************************************/

extern const gm_u32_t gm_crc_table[];

/****************************************************************
 * Macros and inlines
 ****************************************************************/

static gm_inline
unsigned
GM_HASH_PAGE_PORT (gm_up_t page_port)
{
  gm_u32_t c;		/* accumulated crc32 */

#if GM_BUILDING_FIRMWARE
  if (GM_DEBUG_HASH)
    {
      gm_printf ("GM_HASH_PAGE_PORT (0x%qx) called\n", (gm_u64_t) page_port);
    }
#endif

  c = (gm_u32_t) page_port;
  c = (c<<8) ^ gm_crc_table[(c>>24) & 0xff];
  c = (c<<8) ^ gm_crc_table[(c>>24) & 0xff];
  c = (c<<8) ^ gm_crc_table[(c>>24) & 0xff];
  c = (c<<8) ^ gm_crc_table[(c>>24) & 0xff];

#if GM_BUILDING_FIRMWARE
  if (GM_DEBUG_HASH)
    {
      gm_printf ("page_port 0x%qx hashes to 0x%x\n",
          (gm_u64_t) page_port, (unsigned int) c);
    }
#else
  GM_PRINT (GM_DEBUG_HASH,
         ("page_port 0x%lx hashes to 0x%x\n",
          (unsigned long) page_port, (unsigned int) c));
#endif

  return c;
}

#if GM_ENABLE_GALVANTECH_WORKAROUND
static gm_inline
gm_u32_t
gm_galvantech_PTE_checksum (gm_pte_t *_e)
{
  gm_u32_t accum;
  gm_u32_t *iptr = (gm_u32_t *)_e;
  gm_u32_t i0,i1,i2,i3;

  i0 = gm_ntohl(iptr[0]);
  i1 = gm_ntohl(iptr[1]);
  i2 = gm_ntohl(iptr[2]);
  accum = i0+i1;
  i3 = gm_ntohl(iptr[3]);
  accum += i2;
  accum += i3;
  return(accum);
}
#endif /* GALVANTECH */

/****************************************************************
 * function prototypes
 ****************************************************************/

/****************
 * MCP
 ****************/

#if GM_MCP
#define flush_page_hash_cache gm_init_page_table
void gm_init_page_table (void);
gm_dp_t get_uncached_dma_page_entry (gm_up_t page_port);
void gm_fill_pte_from_host (gm_cached_pte_t *entry, unsigned int index);
#endif /* GM_MCP */

/****************
 * kernel
 ****************/

#if GM_KERNEL
gm_dp_t gm_dma_addr_for_mapping (gm_instance_state_t * is, gm_up_t user_vma,
				 gm_u32_t port);
#endif /* GM_KERNEL */


#endif /* _GM_page_table_h_ */

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/include/gm_page_hash.h <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation			 *
 * Copyright (c) 1999 by Myricom, Inc.					 *
 * All rights reserved.	 See the file `COPYING' for copyright notice.	 *
 *************************************************************************/

#ifndef _gm_page_table_h_
#define _gm_page_table_h_

/****************************************************************
 * Switches
 ****************************************************************/

#define GM_DEBUG_PAGE_HASH 0
#define GM_DEBUG_HASH 0

/****************************************************************
 * Includes
 ****************************************************************/

#include "gm.h"
#include "gm_call_trace.h"
#include "gm_crc32.h"
#include "gm_debug.h"

/****************************************************************
 * Globals
 ****************************************************************/

/****************************************************************
 * Macros and inlines
 ****************************************************************/

static gm_inline
unsigned
GM_HASH_PAGE_PORT (gm_up_t page_port)
{
  gm_u32_t c;		/* accumulated crc32 */

  GM_CALLED_WITH_ARGS_EX
    (GM_BUILDING_FIRMWARE && GM_DEBUG_HASH,
     ("GM_HASH_PAGE_PORT (0x%qx) called\n", (gm_u64_t) page_port));

#if GM_BUILDING_FIRMWARE
  /* It is safe to use the global hardware implementation (or the
     software emulation of it) to compute the CRC-32. */
  GM_CRC32_SET (0xffffffffUL);
  if (sizeof (gm_up_t) == 8)
    {
      GM_CRC32_WORD ((gm_u32_t) (page_port >> (8 * sizeof (gm_up_t) / 2)));
    }
  GM_CRC32_WORD ((gm_u32_t) page_port);
  c = GM_CRC32_GET ();
#else
  /* In the host, we use the software implementation that is confirmed to
     agree with the hardware (and the AAL-5 test vectors). */
  c = 0xffffffffUL;
  if (sizeof (gm_up_t) == 8)
    {
      gm_crc32_u32 ((gm_u32_t) (page_port >> (8 * sizeof (gm_up_t) / 2)), &c);
    }
  gm_crc32_u32 ((gm_u32_t) page_port, &c);
#endif
  /* We skip the useless (to us) "c = ~c" CRC step. */
  
#if GM_BUILDING_FIRMWARE
  GM_PRINT
    (GM_DEBUG_HASH,
     ("page_port 0x%qx hashes to 0x%x\n",
      (gm_u64_t) page_port, (unsigned int) c));
#else
  GM_PRINT (GM_DEBUG_HASH,
         ("page_port 0x%lx hashes to 0x%x\n",
          (unsigned long) page_port, (unsigned int) c));
#endif

  GM_RETURN (c);
}

/****************************************************************
 * function prototypes
 ****************************************************************/

/****************
 * kernel
 ****************/

#if GM_KERNEL
gm_dp_t gm_dma_addr_for_mapping (gm_instance_state_t * is, gm_up_t user_vma,
				 gm_u32_t port);
#endif /* GM_KERNEL */


#endif /* _gm_page_table_h_ */
